## Author:  Xu Zhang
## Date:    September 25th 2017
## Program: This code produces the 2nd tropical PCA of apicomplexa dataset in 
##          the heuristic algorithm.
## Input:   Apicomplexa dataset (If you want to apply a different dataset, then
##          you'll need to change the path to read the data and adjust 
##          "sum_orig"; if you want a different order of PC, you need to change
##          "pcs". In this code, pcs is equal to 3 which is the 2nd PC.)
## Output:  "apicom_output_approx.txt", "proj_api_topology.pdf" and the summary
##          r statistic.


require(Matrix)
require(ape)
library(phangorn)
library(lpSolveAPI)
library(parallel)

setwd("~/Dropbox/rudy_xu/sim gene under species")

#function to generate random trees
dist_vector<-function(k,m){
  vecTreesArray<-array(NA,dim = c(k,choose(m,2))) 
  tree_store<-list()
  for(p in 1:k){
    t <- rcoal(m) ### generate an equidistant tree 
    tree_store[[p]]<-t
    t$tip.label<-seq(1,m,by=1)  ###change the labels
    
    D1 <- cophenetic(t) ### Compute the distant matrix
    
    ###vectorize the distance matrix
    for(row.num in 1:(m-1)){
      for(col.num in (row.num+1):m){
        vecTreesArray[p,col.num-row.num+(m-1+(m-1-row.num+2))*(row.num-1)/2]<-D1[row.num,col.num]  
      }
    }   
  }
  return(list(vecTreesArray,tree_store))
}

#function to get tropical distance of two points
tropical_dist<-function(D_1,D_2){
  e<-length(D_1)
  t_dist<-0
  for(i in 1:(e-1)){
    for(j in (i+1):e){
      if(abs(D_1[i]-D_2[i]-D_1[j]+D_2[j])>t_dist){
        t_dist<-abs(D_1[i]-D_2[i]-D_1[j]+D_2[j])
      }
    }
  }
  t_dist
}   

#function to project tree D onto tconv(D_s) with s PC's and m leaves
project_pi<-function(s,m,D_s,D){
  lambda<-rep(NA,s)
  for(i in 1:s){
    lambda[i]<-min(D-D_s[i,])#D_s by row
  }
  
  pi_D<-rep(NA,choose(m,2))
  for(i in 1:choose(m,2)){
    pi_D[i]<-max(lambda+D_s[,i])
  }
  return(pi_D)
}

#DD<-dist_vector(20,8)
#par(mfrow=c(2,4))
#plot(DD[[2]][[1]])

#class(DD[[2]])<-'multiPhylo'
#write.tree(DD[[2]],file='random_sample_20_8.txt')
#D<-DD[[1]]
#n<-dim(D)[1]
#e<-dim(D)[2]
#data_matrix<-cbind(d1,d2,d3) #by column
#data_matrix<-t(D)
#pcs<-3
#m<-8 #num of leaves

#n<-5
#e<-3
#data_matrix<-matrix(c(1,3,3,0,1,1,0,3,3,3.5,4,4,2,7,7),ncol=n,byrow=F)
#D<-t(data_matrix)
#pcs<-3
###%%%%%%%%%%%%%%%%%

mytree<-read.tree('apicomplexa_non_out.txt')
is.rooted(mytree)
#all TRUE
tr<-root(mytree, outgroup = 'Tt',resolve.root=TRUE)
is.rooted(tr)
N<-length(tr) #252 trees
#chronotrees<-list()
chronotrees<-lapply(tr, chronos)
dist_chrono<-lapply(chronotrees,cophenetic)

#to order the labels
dist_ordered<-list(NA,N)
for(i in 1:N){
  dist_ordered[[i]]<-dist_chrono[[i]][chronotrees[[1]]$tip.label,chronotrees[[1]]$tip.label]
}

vec_fun<-function(x){
  m<-dim(x)[1]
  vecTreesVec<-rep(NA,choose(m,2)) 
  for(row.num in 1:(m-1)){
    for(col.num in (row.num+1):m){
      vecTreesVec[col.num-row.num+(m-1+(m-1-row.num+2))*(row.num-1)/2]<-x[row.num,col.num]  
    }
  }
  vecTreesVec
}

vectorized_dist<-lapply(dist_ordered,vec_fun)
#vectorized_dist2<-vectorized_dist[[1:10]]
n<-N
e<-length(vectorized_dist[[1]])
m<-length(tr[[1]]$tip.label) #leaves = 8
data_matrix<-matrix(unlist(vectorized_dist),ncol=n,byrow=F) 
#data_matrix<-cbind(d1,d2,d3) #by column
pcs<-3
D<-t(data_matrix)


#all possible combinations
sample_set_ind<-combn(n,pcs,simplify = F)
sum_orig<-100000 #initial upper bound for the sum of distances
data_new<-D #not exclude the PC base
data.new.list<-lapply(seq_len(nrow(data_new)), function(j) data_new[j,])
for(i in 1:choose(n,pcs)){
  pc_base<-D[sample_set_ind[[i]],]#by row
  proj_points<-lapply(data.new.list,project_pi,s=pcs,m=m,D_s=pc_base)
  tropical.dist.vec<-mapply(tropical_dist,data.new.list,proj_points)
  sum_dist<-sum(tropical.dist.vec)
  if(sum_dist < sum_orig){
    label_output<-list(pc_base,proj_points,tropical.dist.vec,sum_dist,i)
    sum_orig<-sum_dist
  }
  print(i)
}
names(label_output)<-c("PC's","Projected points","Dist","Sum of dist","i")

sink("apicom_output_approx.txt")
print(label_output)
sink()


###From Rudy###########
#source("tropical_tree_triangle.R")
#source("randon_tree_ultrametric3.R")
#api_pca<-label_output$`PC's`
#DD<-api_pca
#D_api<-normalize.ultrametrices(DD)
#plot.trop.triangle(D_api)


####Count the frequency of topology of projected trees

make.matrix <- function(D, n, tips){
  dd <- matrix(rep(0, n*n), nrow=n, ncol=n, byrow=TRUE)
  count <- 1
  for(i in 1:(n-1))
    for(j in (i + 1):n){
      dd[i, j] <- D[count]
      dd[j, i] <- dd[i, j]
      count <- count+1
    }
  mymatrix <- matrix(dd, nrow=n, ncol=n, byrow=TRUE, dimnames=list(tips,tips))
  return(mymatrix)
}

numvectors <- dim(data_matrix)[2]

adj_proj_points<-label_output$`Projected points`
adj_proj_points[[2]]<-adj_proj_points[[2]]+1
adj_proj_points[[3]]<-adj_proj_points[[3]]+1
adj_proj_points[[14]]<-adj_proj_points[[14]]+1
adj_proj_points[[18]]<-adj_proj_points[[18]]+1
adj_proj_points[[24]]<-adj_proj_points[[24]]+1
#proj_trees_matrix<-lapply(label_output$`Projected points`,make.matrix,
 #                  n=m,tips=chronotrees[[1]]$tip.label)
proj_trees_matrix<-lapply(adj_proj_points,make.matrix,
                          n=m,tips=chronotrees[[1]]$tip.label)
proj_trees<-lapply(proj_trees_matrix,upgma)
type <- seq(numvectors)
for(i in seq(numvectors)){
  if(i==type[i]) {
    for(j in seq(i, numvectors)) {
      if(RF.dist(proj_trees[[i]],proj_trees[[j]])==0) {
        type[j]=i
      }
    } 
  }
}

table(type)

#type
#1   2   3  14  18  24  85 106 
#71  50  70  21  27   8   4   1 

pdf("proj_api_topology.pdf",width = 5, height = 10)
par(mfrow=c(4,2))
plot(proj_trees[[1]],main="(71)")
plot(proj_trees[[2]],main="(50)")
plot(proj_trees[[3]],main="(70)")
plot(proj_trees[[14]],main="(21)")
plot(proj_trees[[18]],main="(27)")
plot(proj_trees[[24]],main="(8)")
plot(proj_trees[[85]],main="(4)")
plot(proj_trees[[106]],main="(1)")
dev.off()


#r^2########
library(lpSolveAPI)
library(parallel)
fermatweberdistance <- function(datamatrix) {
  n = dim(datamatrix)[1]
  m = dim(datamatrix)[2]
  lprec <- make.lp(0, n+m)
  objective = mat.or.vec(n+m,1)
  for (i in seq(n)) {
    objective[i] = 1
  }
  set.objfn(lprec, objective)
  for (i in seq(n)) {
    for (j in seq(m)) {
      for (k in seq(m)) {
        v = mat.or.vec(n+m,1)
        v[i] = 1
        v[n+k] = 1
        v[n+j] = -1
        add.constraint(lprec, v, ">=", datamatrix[i,k] - datamatrix[i,j])
      }
    }
  }
  solve(lprec)
  return((get.objective(lprec)))
}

r_proj_data<-matrix(unlist(label_output$`Projected points`),nrow=252,byrow = T)

apicom_fermet<-fermatweberdistance(r_proj_data)
#231.7858

#sum of dist = 147.0568
r_apicom<-label_output$`Sum of dist`/(label_output$`Sum of dist`+apicom_fermet)
#0.388174
